var Sup09500_ComparativoEntreCustos_Controller = function(ZhFilterPreferences, ZhLayoutPreferences, Utilities, templateManager, maskEngine, ZhUtil) {

    // It sets Default values for the field and open filter
    this.onOpenFilter = function(widget) {
        let rowsToSet = {};
        const date = moment().format('L');
        const costField = widget.getField('NMTIPOCUSTO');
        const buyerField = widget.getField('CDOPERADOR');

        // realoads are needed to load dataSource.data fields previously
        costField.reload().then(_ => {
            const { CDTIPOCUSTO, NMTIPOCUSTO } = costField.dataSource.data[0];
            buyerField.reload().then(_ => {
                buyerField.dataSource.data.forEach(buyer => {
                    if (buyer.CDOPERADOR == getLocalVar('CDOPERADOR'))
                        rowsToSet.CDOPERADOR = [buyer.CDOPERADOR];
                });

                rowsToSet.DTCONTAGEM = { "start": date, "end": date, };
                rowsToSet.NMRELATORIO = 'SUP09500';
                rowsToSet.CDTIPOCUSTO = CDTIPOCUSTO;
                rowsToSet.NMTIPOCUSTO = NMTIPOCUSTO;
                rowsToSet.CDREGIAO = '';
                rowsToSet.CDGRUPFILI = '';

                ZhFilterPreferences.openFilterWidget(widget, rowsToSet);
            });
        });
    };

    // It sets formated values in the filter to exceute the main query
    this.setFilterParams = function(widget, preference = null) {
        let filtro = [];
        const filterRow = preference.FILTER_ROW;

        filtro = [
            {
                "name": "DTINICIAL",
                "operator": "=",
                "value": filterRow.DTCONTAGEM ? filterRow.DTCONTAGEM.start : null
            },
            {
                "name": "DTFINAL",
                "operator": "=",
                "value": filterRow.DTCONTAGEM ? filterRow.DTCONTAGEM.end : null
            },
            {
                "name": "CDTIPOCUSTO",
                "operator": "=",
                "value": filterRow.CDTIPOCUSTO
            },
            {
                "name": "CDFILDEST",
                "operator": "=",
                "value": this.onFormatMutipleFields(filterRow.CDFILDEST)
            },
            {
                "name": "CDREGIAO",
                "operator": "=",
                "value": this.onFormatMutipleFields(filterRow.CDREGIAO)
            },
            {
                "name": "CDGRUPFILI",
                "operator": "=",
                "value": this.onFormatMutipleFields(filterRow.CDGRUPFILI)
            },
            {
                "name": "CDCOMPRADOR",
                "operator": "=",
                "value": this.onFormatMutipleFields(filterRow.CDOPERADOR)
            },
            {
                "name": "CDFORNECED",
                "operator": "=",
                "value": this.onFormatMutipleFields(filterRow.NRINSJURFORN)
            },
            {
                "name": "CDPRODINI",
                "operator": "=",
                "value": filterRow.CDPRODINI
            },
            {
                "name": "CDPRODFIN",
                "operator": "=",
                "value": filterRow.CDPRODFIN ? filterRow.CDPRODFIN.padEnd(20, 'z') : null
            }
        ];

        widget.dataSourceFilter = filtro;
        widget.reload();
    };

    // This sets formated values to dataSourceFilter of the unity field; 
    this.UnityOnClick = function(field) {
        const { CDGRUPFILI, CDREGIAO } = field.widget.currentRow;

        field.dataSourceFilter = [
            {
                "name": "CDREGIAO",
                "operator": "=",
                "value": CDREGIAO != '' ? this.onFormatMutipleFields(CDREGIAO) : ''
            },
            {
                "name": "CDGRUPFILI",
                "operator": "=",
                "value": CDGRUPFILI != '' ? this.onFormatMutipleFields(CDGRUPFILI) : ''
            }
        ];
        field.reload();
    };

    // General function to format mutiple fields values Ex: [01,02] becomes _01_02_
    this.onFormatMutipleFields = function(arrValue = []) {
        if (arrValue.length > 0) {
            let FormatedToString = arrValue.toString().replaceAll(',', '_');
            return '_' + FormatedToString + '_';
        }
        else {
            return null;
        }
    };

    // this function manage metadata of reports
    this.reportProcess = function(args, afterOpenReport = false) {
        const { owner: widget } = args;
        if (afterOpenReport) {
            ZhLayoutPreferences.applyOriginalLayout(widget);
        }
        else {
            const metadata = args.e.metadata;
            const DTCONTAGEM = Utilities.getFieldValue('DTCONTAGEM', widget.name, true);
            const CDREGIAO = Utilities.getFieldValue('CDREGIAO', widget.name, true);
            const NMREGIAO = Utilities.getFieldValue('NMREGIAO', widget.name, true);
            const CDPRODINI = Utilities.getFieldValue('CDPRODINI', widget.name, true);
            const CDPRODFIN = Utilities.getFieldValue('CDPRODFIN', widget.name, true);
            const NMTIPOCUSTO = Utilities.getFieldValue('NMTIPOCUSTO', widget.name, true);

            let filter = [];

            let regiaoValue = this.formatReportHeaderValues('formatRegiao', { 'CDREGIAO': CDREGIAO, 'NMREGIAO': NMREGIAO });

            if (regiaoValue) {
                filter.push({
                    label: "Região",
                    value: regiaoValue
                });
            }

            filter.push({
                label: "Período",
                value: this.formatReportHeaderValues('formatDate', { 'DTCONTAGEM': DTCONTAGEM })
            });

            let produtoValue = this.formatReportHeaderValues('formatProduto', { 'CDPRODINI': CDPRODINI, 'CDPRODFIN': CDPRODFIN });

            if (produtoValue) {
                filter.push({
                    label: "Produto",
                    value: produtoValue
                });
            }

            filter.push({
                label: "Tipo de Custo",
                value: NMTIPOCUSTO
            });

            metadata.filter = filter;
            metadata.orientation = 'LANDSCAPE';
            ZhLayoutPreferences.applyOriginalLayout(widget);
        }
    };

    // This function will format all values passed, to show in header Values
    this.formatReportHeaderValues = function(type, objValues = {}) {
        let result = '';
        switch (type) {
            case 'formatRegiao':
                {
                    let { CDREGIAO, NMREGIAO } = objValues;

                    if (CDREGIAO && NMREGIAO) {
                        for (let [indexCode, valueCode] of CDREGIAO.entries()) {
                            for (let [indexName, valueName] of NMREGIAO.entries()) {
                                if (indexCode == indexName)
                                    result = result + `${valueCode} - ${valueName},  `;
                            }
                        }
                    }
                    else {
                        result = false;
                    }
                    break;
                }
            case 'formatDate':
                {
                    let { start, end } = objValues.DTCONTAGEM;

                    result = `${start} à ${end}`;
                    break;
                }
            case 'formatProduto':
                {
                    let { CDPRODINI, CDPRODFIN } = objValues;

                    let mask = {
                        type: "fix",
                        params: {
                            mask: getLocalVar('CDPICTPROD')
                        }
                    };
                    CDPRODINI = Util.applyMask(CDPRODINI, mask, maskEngine, ZhUtil).split('.').toString().replaceAll(',', '.');
                    CDPRODFIN = Util.applyMask(CDPRODFIN, mask, maskEngine, ZhUtil).split('.').toString().replaceAll(',', '.');

                    result = CDPRODINI && CDPRODFIN ? `${CDPRODINI} à ${CDPRODFIN}` : false;
                    break;
                }
        }
        return result;
    };

    // This function will Set dataSourceFilter of tipo custo field
    this.setFilterTipoCusto = function() {
        const language = getLocalVar('language');
        const valIdioma = {
            'es_cl': 'E',
            'en_us': 'I',
            'pt_br': 'B'
        };
        return valIdioma[language] || 'B';
    };
};
Configuration(function(ContextRegister) {
    ContextRegister.register("Sup09500_ComparativoEntreCustos_Controller", Sup09500_ComparativoEntreCustos_Controller);
});
